@@ -9,7 +9,7 @@
   /**
    * @unrestricted
    */
-  var DevToolsAPIImpl = class {
+  const DevToolsAPIImpl = class {
     constructor() {
       /**
        * @type {number}
@@ -27,7 +27,7 @@
      * @param {?Object} arg
      */
     embedderMessageAck(id, arg) {
-      var callback = this._callbacks[id];
+      const callback = this._callbacks[id];
       delete this._callbacks[id];
       if (callback)
         callback(arg);
@@ -39,10 +39,10 @@
      * @param {?function(?Object)} callback
      */
     sendMessageToEmbedder(method, args, callback) {
-      var callId = ++this._lastCallId;
+      const callId = ++this._lastCallId;
       if (callback)
         this._callbacks[callId] = callback;
-      var message = {'id': callId, 'method': method};
+      const message = {'id': callId, 'method': method};
       if (args.length)
         message.params = args;
       DevToolsHost.sendMessageToEmbedder(JSON.stringify(message));
@@ -297,7 +297,7 @@
      * @return {string}
      */
     _decodeBase64(chunk) {
-      var request = new XMLHttpRequest();
+      const request = new XMLHttpRequest();
       request.open('GET', 'data:text/plain;base64,' + chunk, false);
       request.send(null);
       if (request.status === 200) {
@@ -309,7 +309,7 @@
     }
   };
 
-  var DevToolsAPI = new DevToolsAPIImpl();
+  const DevToolsAPI = new DevToolsAPIImpl();
   window.DevToolsAPI = DevToolsAPI;
 
   // InspectorFrontendHostImpl --------------------------------------------------
@@ -318,37 +318,33 @@
    * @implements {InspectorFrontendHostAPI}
    * @unrestricted
    */
-  var InspectorFrontendHostImpl = class {
+  const InspectorFrontendHostImpl = class {
     /**
-     * @override
      * @return {string}
      */
     getSelectionBackgroundColor() {
-      return DevToolsHost.getSelectionBackgroundColor();
+      return '#6e86ff';
     }
 
     /**
-     * @override
      * @return {string}
      */
     getSelectionForegroundColor() {
-      return DevToolsHost.getSelectionForegroundColor();
+      return '#ffffff';
     }
 
     /**
-     * @override
      * @return {string}
      */
     getInactiveSelectionBackgroundColor() {
-      return DevToolsHost.getInactiveSelectionBackgroundColor();
+      return '#c9c8c8';
     }
 
     /**
-     * @override
      * @return {string}
      */
     getInactiveSelectionForegroundColor() {
-      return DevToolsHost.getInactiveSelectionForegroundColor();
+      return '#323232';
     }
 
     /**
@@ -366,7 +362,7 @@
       DevToolsAPI.sendMessageToEmbedder('loadCompleted', [], null);
       // Support for legacy (<57) frontends.
       if (window.Runtime && window.Runtime.queryParam) {
-        var panelToOpen = window.Runtime.queryParam('panel');
+        const panelToOpen = window.Runtime.queryParam('panel');
         if (panelToOpen)
           window.DevToolsAPI.showPanel(panelToOpen);
       }
@@ -515,6 +511,13 @@
       DevToolsAPI.sendMessageToEmbedder('append', [url, content], null);
     }
 
+    /**
+     * @override
+     * @param {string} url
+     */
+    close(url) {
+    }
+
     /**
      * @override
      * @param {string} message
@@ -581,9 +584,13 @@
      * @override
      * @param {number} requestId
      * @param {string} fileSystemPath
+     * @param {string} excludedFolders
      */
-    indexPath(requestId, fileSystemPath) {
-      DevToolsAPI.sendMessageToEmbedder('indexPath', [requestId, fileSystemPath], null);
+    indexPath(requestId, fileSystemPath, excludedFolders) {
+      // |excludedFolders| added in M67. For backward compatibility,
+      // pass empty array.
+      excludedFolders = excludedFolders || '[]';
+      DevToolsAPI.sendMessageToEmbedder('indexPath', [requestId, fileSystemPath, excludedFolders], null);
     }
 
     /**
@@ -807,59 +814,6 @@
       return true;
     }
 
-    /**
-     * Support for legacy front-ends (<M28).
-     * @return {boolean}
-     */
-    canInspectWorkers() {
-      return true;
-    }
-
-    /**
-     * Support for legacy front-ends (<M28).
-     * @return {boolean}
-     */
-    canSaveAs() {
-      return true;
-    }
-
-    /**
-     * Support for legacy front-ends (<M28).
-     * @return {boolean}
-     */
-    canSave() {
-      return true;
-    }
-
-    /**
-     * Support for legacy front-ends (<M28).
-     */
-    loaded() {
-    }
-
-    /**
-     * Support for legacy front-ends (<M28).
-     * @return {string}
-     */
-    hiddenPanels() {
-      return '';
-    }
-
-    /**
-     * Support for legacy front-ends (<M28).
-     * @return {string}
-     */
-    localizedStringsURL() {
-      return '';
-    }
-
-    /**
-     * Support for legacy front-ends (<M28).
-     * @param {string} url
-     */
-    close(url) {
-    }
-
     /**
      * Support for legacy front-ends (<M44).
      * @param {number} actionCode
@@ -883,7 +837,7 @@
 
   function installObjectObserve() {
     /** @type {!Array<string>} */
-    var properties = [
+    const properties = [
       'advancedSearchConfig',
       'auditsPanelSplitViewState',
       'auditsSidebarWidth',
@@ -1068,13 +1022,13 @@
      */
     function objectObserve(object, observer) {
       if (window['WebInspector']) {
-        var settingPrototype = /** @type {!Object} */ (window['WebInspector']['Setting']['prototype']);
+        const settingPrototype = /** @type {!Object} */ (window['WebInspector']['Setting']['prototype']);
         if (typeof settingPrototype['remove'] === 'function')
           settingPrototype['remove'] = settingRemove;
       }
       /** @type {!Set<string>} */
-      var changedProperties = new Set();
-      var scheduled = false;
+      const changedProperties = new Set();
+      let scheduled = false;
 
       function scheduleObserver() {
         if (scheduled)
@@ -1085,7 +1039,7 @@
 
       function callObserver() {
         scheduled = false;
-        var changes = /** @type {!Array<!{name: string}>} */ ([]);
+        const changes = /** @type {!Array<!{name: string}>} */ ([]);
         changedProperties.forEach(function(name) {
           changes.push({name: name});
         });
@@ -1094,7 +1048,7 @@
       }
 
       /** @type {!Map<string, *>} */
-      var storage = new Map();
+      const storage = new Map();
 
       /**
        * @param {string} property
@@ -1124,7 +1078,7 @@
         });
       }
 
-      for (var i = 0; i < properties.length; ++i)
+      for (let i = 0; i < properties.length; ++i)
         defineProperty(properties[i]);
     }
 
@@ -1132,7 +1086,7 @@
   }
 
   /** @type {!Map<number, string>} */
-  var staticKeyIdentifiers = new Map([
+  const staticKeyIdentifiers = new Map([
     [0x12, 'Alt'],
     [0x11, 'Control'],
     [0x10, 'Shift'],
@@ -1196,23 +1150,148 @@
    * @return {string}
    */
   function keyCodeToKeyIdentifier(keyCode) {
-    var result = staticKeyIdentifiers.get(keyCode);
+    let result = staticKeyIdentifiers.get(keyCode);
     if (result !== undefined)
       return result;
     result = 'U+';
-    var hexString = keyCode.toString(16).toUpperCase();
-    for (var i = hexString.length; i < 4; ++i)
+    const hexString = keyCode.toString(16).toUpperCase();
+    for (let i = hexString.length; i < 4; ++i)
       result += '0';
     result += hexString;
     return result;
   }
 
   function installBackwardsCompatibility() {
-    if (window.location.search.indexOf('remoteFrontend') === -1)
+    const majorVersion = getRemoteMajorVersion();
+    if (!majorVersion)
       return;
 
-    // Support for legacy (<M53) frontends.
-    if (!window.KeyboardEvent.prototype.hasOwnProperty('keyIdentifier')) {
+    /** @type {!Array<string>} */
+    const styleRules = [];
+    // Shadow DOM V0 polyfill
+    if (majorVersion <= 73 && !Element.prototype.createShadowRoot) {
+      Element.prototype.createShadowRoot = function() {
+        try {
+          return this.attachShadow({mode: 'open'});
+        } catch (e) {
+          // some elements we use to add shadow roots can no
+          // longer have shadow roots.
+          const fakeShadowHost = document.createElement('span');
+          this.appendChild(fakeShadowHost);
+          fakeShadowHost.className = 'fake-shadow-host';
+          return fakeShadowHost.createShadowRoot();
+        }
+      };
+
+      const origAdd = DOMTokenList.prototype.add;
+      DOMTokenList.prototype.add = function(...tokens) {
+        if (tokens[0].startsWith('insertion-point') || tokens[0].startsWith('tabbed-pane-header'))
+          this._myElement.slot = '.' + tokens[0];
+        return origAdd.apply(this, tokens);
+      };
+
+      const origCreateElement = Document.prototype.createElement;
+      Document.prototype.createElement = function(tagName, ...rest) {
+        if (tagName === 'content')
+          tagName = 'slot';
+        const element = origCreateElement.call(this, tagName, ...rest);
+        element.classList._myElement = element;
+        return element;
+      };
+
+      Object.defineProperty(HTMLSlotElement.prototype, 'select', {
+        async set(selector) {
+          this.name = selector;
+        }
+      });
+
+      // Document.prototype.createElementWithClass is a DevTools method, so we
+      // need to wait for DOMContentLoaded in order to override it.
+      if (window.document.head &&
+          (window.document.readyState === 'complete' || window.document.readyState === 'interactive'))
+        overrideCreateElementWithClass();
+      else
+        window.addEventListener('DOMContentLoaded', overrideCreateElementWithClass);
+
+      function overrideCreateElementWithClass() {
+        window.removeEventListener('DOMContentLoaded', overrideCreateElementWithClass);
+
+        const origCreateElementWithClass = Document.prototype.createElementWithClass;
+        Document.prototype.createElementWithClass = function(tagName, className, ...rest) {
+          if (tagName !== 'button' || (className !== 'soft-dropdown' && className !== 'dropdown-button'))
+            return origCreateElementWithClass.call(this, tagName, className, ...rest);
+          const element = origCreateElementWithClass.call(this, 'div', className, ...rest);
+          element.tabIndex = 0;
+          element.role = 'button';
+          return element;
+        };
+      }
+    }
+
+    // Custom Elements V0 polyfill
+    if (majorVersion <= 73 && !Document.prototype.registerElement) {
+      const fakeRegistry = new Map();
+      Document.prototype.registerElement = function(typeExtension, options) {
+        const {prototype, extends: localName} = options;
+        const document = this;
+        const callback = function() {
+          const element = document.createElement(localName || typeExtension);
+          const skip = new Set(['constructor', '__proto__']);
+          for (const key of Object.keys(Object.getOwnPropertyDescriptors(prototype.__proto__ || {}))) {
+            if (skip.has(key))
+              continue;
+            element[key] = prototype[key];
+          }
+          element.setAttribute('is', typeExtension);
+          if (element['createdCallback'])
+            element['createdCallback']();
+          return element;
+        };
+        fakeRegistry.set(typeExtension, callback);
+        return callback;
+      };
+
+      const origCreateElement = Document.prototype.createElement;
+      Document.prototype.createElement = function(tagName, fakeCustomElementType) {
+        const fakeConstructor = fakeRegistry.get(fakeCustomElementType);
+        if (fakeConstructor)
+          return fakeConstructor();
+        return origCreateElement.call(this, tagName, fakeCustomElementType);
+      };
+
+      // DevTools front-ends mistakenly assume that
+      //   classList.toggle('a', undefined) works as
+      //   classList.toggle('a', false) rather than as
+      //   classList.toggle('a');
+      const originalDOMTokenListToggle = DOMTokenList.prototype.toggle;
+      DOMTokenList.prototype.toggle = function(token, force) {
+        if (arguments.length === 1)
+          force = !this.contains(token);
+        return originalDOMTokenListToggle.call(this, token, !!force);
+      };
+    }
+
+    if (majorVersion <= 66) {
+      /** @type {(!function(number, number):Element|undefined)} */
+      ShadowRoot.prototype.__originalShadowRootElementFromPoint;
+
+      if (!ShadowRoot.prototype.__originalShadowRootElementFromPoint) {
+        ShadowRoot.prototype.__originalShadowRootElementFromPoint = ShadowRoot.prototype.elementFromPoint;
+        /**
+         *  @param {number} x
+         *  @param {number} y
+         *  @return {Element}
+         */
+        ShadowRoot.prototype.elementFromPoint = function(x, y) {
+          const originalResult = ShadowRoot.prototype.__originalShadowRootElementFromPoint.apply(this, arguments);
+          if (this.host && originalResult === this.host)
+            return null;
+          return originalResult;
+        };
+      }
+    }
+
+    if (majorVersion <= 53) {
       Object.defineProperty(window.KeyboardEvent.prototype, 'keyIdentifier', {
         /**
          * @return {string}
@@ -1224,91 +1303,114 @@
       });
     }
 
-    // Support for legacy (<M50) frontends.
-    installObjectObserve();
+    if (majorVersion <= 50)
+      installObjectObserve();
 
-    /**
-     * @param {string} property
-     * @return {!CSSValue|null}
-     * @this {CSSStyleDeclaration}
-     */
-    function getValue(property) {
-      // Note that |property| comes from another context, so we can't use === here.
-      // eslint-disable-next-line eqeqeq
-      if (property == 'padding-left') {
-        return /** @type {!CSSValue} */ ({
-          /**
-           * @return {number}
-           * @this {!{__paddingLeft: number}}
-           */
-          getFloatValue: function() {
-            return this.__paddingLeft;
-          },
-          __paddingLeft: parseFloat(this.paddingLeft)
-        });
+    if (majorVersion <= 45) {
+      /**
+       * @param {string} property
+       * @return {!CSSValue|null}
+       * @this {CSSStyleDeclaration}
+       */
+      function getValue(property) {
+        // Note that |property| comes from another context, so we can't use === here.
+        // eslint-disable-next-line eqeqeq
+        if (property == 'padding-left') {
+          return /** @type {!CSSValue} */ ({
+            /**
+             * @return {number}
+             * @this {!{__paddingLeft: number}}
+             */
+            getFloatValue: function() {
+              return this.__paddingLeft;
+            },
+            __paddingLeft: parseFloat(this.paddingLeft)
+          });
+        }
+        throw new Error('getPropertyCSSValue is undefined');
       }
-      throw new Error('getPropertyCSSValue is undefined');
+
+      window.CSSStyleDeclaration.prototype.getPropertyCSSValue = getValue;
+
+      function CSSPrimitiveValue() {
+      }
+      CSSPrimitiveValue.CSS_PX = 5;
+      window.CSSPrimitiveValue = CSSPrimitiveValue;
     }
 
-    // Support for legacy (<M41) frontends.
-    window.CSSStyleDeclaration.prototype.getPropertyCSSValue = getValue;
+    if (majorVersion <= 45)
+      styleRules.push('* { min-width: 0; min-height: 0; }');
 
-    function CSSPrimitiveValue() {
+    if (majorVersion <= 51) {
+      // Support for quirky border-image behavior (<M51), see:
+      // https://bugs.chromium.org/p/chromium/issues/detail?id=559258
+      styleRules.push('.cm-breakpoint .CodeMirror-linenumber { border-style: solid !important; }');
+      styleRules.push(
+          '.cm-breakpoint.cm-breakpoint-conditional .CodeMirror-linenumber { border-style: solid !important; }');
+    }
+    if (majorVersion <= 71) {
+      styleRules.push(
+          '.coverage-toolbar-container, .animation-timeline-toolbar-container, .computed-properties { flex-basis: auto; }');
     }
-    CSSPrimitiveValue.CSS_PX = 5;
-    window.CSSPrimitiveValue = CSSPrimitiveValue;
 
-    // Support for legacy (<M44) frontends.
-    var styleElement = window.document.createElement('style');
-    styleElement.type = 'text/css';
-    styleElement.textContent = 'html /deep/ * { min-width: 0; min-height: 0; }';
+    if (majorVersion <= 50)
+      Event.prototype.deepPath = undefined;
 
-    // Support for quirky border-image behavior (<M51), see:
-    // https://bugs.chromium.org/p/chromium/issues/detail?id=559258
-    styleElement.textContent +=
-        '\nhtml /deep/ .cm-breakpoint .CodeMirror-linenumber { border-style: solid !important; }';
-    styleElement.textContent +=
-        '\nhtml /deep/ .cm-breakpoint.cm-breakpoint-conditional .CodeMirror-linenumber { border-style: solid !important; }';
-    window.document.head.appendChild(styleElement);
+    if (majorVersion <= 54) {
+      window.FileError = /** @type {!function (new: FileError) : ?} */ ({
+        NOT_FOUND_ERR: DOMException.NOT_FOUND_ERR,
+        ABORT_ERR: DOMException.ABORT_ERR,
+        INVALID_MODIFICATION_ERR: DOMException.INVALID_MODIFICATION_ERR,
+        NOT_READABLE_ERR: 0  // No matching DOMException, so code will be 0.
+      });
+    }
 
-    // Support for legacy (<M49) frontends.
-    Event.prototype.deepPath = undefined;
-
-    // Support for legacy (<53) frontends.
-    window.FileError = /** @type {!function (new: FileError) : ?} */ ({
-      NOT_FOUND_ERR: DOMException.NOT_FOUND_ERR,
-      ABORT_ERR: DOMException.ABORT_ERR,
-      INVALID_MODIFICATION_ERR: DOMException.INVALID_MODIFICATION_ERR,
-      NOT_READABLE_ERR: 0  // No matching DOMException, so code will be 0.
-    });
+    installExtraStyleRules(styleRules);
   }
 
-  function windowLoaded() {
-    window.removeEventListener('DOMContentLoaded', windowLoaded, false);
-    installBackwardsCompatibility();
+  /**
+   * @return {?number}
+   */
+  function getRemoteMajorVersion() {
+    try {
+      const remoteVersion = new URLSearchParams(window.location.search).get('remoteVersion');
+      if (!remoteVersion)
+        return null;
+      const majorVersion = parseInt(remoteVersion.split('.')[0], 10);
+      return majorVersion;
+    } catch (e) {
+      return null;
+    }
   }
 
-  if (window.document.head &&
-      (window.document.readyState === 'complete' || window.document.readyState === 'interactive'))
-    installBackwardsCompatibility();
-  else
-    window.addEventListener('DOMContentLoaded', windowLoaded, false);
+  /**
+   * @param {!Array<string>} styleRules
+   */
+  function installExtraStyleRules(styleRules) {
+    if (!styleRules.length)
+      return;
+    const styleText = styleRules.join('\n');
+    document.head.appendChild(createStyleElement(styleText));
 
-  /** @type {(!function(string, boolean=):boolean)|undefined} */
-  DOMTokenList.prototype.__originalDOMTokenListToggle;
-
-  if (!DOMTokenList.prototype.__originalDOMTokenListToggle) {
-    DOMTokenList.prototype.__originalDOMTokenListToggle = DOMTokenList.prototype.toggle;
-    /**
-     * @param {string} token
-     * @param {boolean=} force
-     * @return {boolean}
-     */
-    DOMTokenList.prototype.toggle = function(token, force) {
-      if (arguments.length === 1)
-        force = !this.contains(token);
-      return this.__originalDOMTokenListToggle(token, !!force);
+    const origCreateShadowRoot = HTMLElement.prototype.createShadowRoot;
+    HTMLElement.prototype.createShadowRoot = function(...args) {
+      const shadowRoot = origCreateShadowRoot.call(this, ...args);
+      shadowRoot.appendChild(createStyleElement(styleText));
+      return shadowRoot;
     };
   }
 
+  /**
+   * @param {string} styleText
+   * @return {!Element}
+   */
+  function createStyleElement(styleText) {
+    const style = document.createElement('style');
+    style.type = 'text/css';
+    style.textContent = styleText;
+    return style;
+  }
+
+  installBackwardsCompatibility();
+
 })(window);
